package de.digisalt.tileEdit.model.dataContainers;

import java.util.Random;

import de.digisalt.tileEdit.controller.Commands;
import de.digisalt.tileEdit.controller.Controller;

public class TileContainer {
	int tile[][] = new int[8][8];
	
	public TileContainer(boolean randomTest)
	{
		Random gen = new Random();
		for (int i=0; i<=7; i++)
		{
			for (int j=0; j<=7; j++)
			{
				if (randomTest)
				{
					tile[i][j] = Math.abs(gen.nextInt() % 15);					
				}
				else
				{
					tile[i][j] = 0;
				}
			}
		}		
	}
	
	public boolean importColorValues(int[] in)
	{
		if (in.length != 64) return false;
		for (int i=0; i<in.length; i++)
		{
			try {
				//System.out.println("x|y|col: " + i%8 + "|"+ (int)Math.floor(i/8) + "|" + in[i]);
				setColorAt(7-i%8, (int)Math.floor(i/8), in[i]);
			} catch (Exception e) {
				Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Error importing color array into tile container"});
				return false;
			}
		}
		return true;
	}
	
	public boolean setColorAt(int x, int y, int color) throws Exception
	{
		if (x > 7 || x < 0 || y > 7 || y < 0 || color < 0 || color > 15)
		{
			throw new Exception();
		}
		tile[x][y] = color;
		return true;
	}
	
	public int getColorAt(int x, int y) throws Exception
	{
		if (x > 7 || x < 0 || y > 7 || y < 0)
		{
			//System.out.println("x|y: " + x + "|" + y);
			throw new Exception();
		}
		return tile[x][y];
	}
	
	public void flipVertical()
	{
		int old[][] = new int[8][8];
		for (int i=0; i<=7; i++)
		{			
			for (int j=0; j<=7; j++)
			{
				old[i][j] = tile[i][j]; 
			}
		}
		
		for (int i=0; i<=7; i++)
		{
			for (int j=0; j<=7; j++)
			{
				tile[i][j] = old[i][7-j];
			}
		}
	}
	
	public void flipHorizontal()
	{
		int old[][] = new int[8][8];
		for (int i=0; i<=7; i++)
		{			
			for (int j=0; j<=7; j++)
			{
				old[i][j] = tile[i][j]; 
			}
		}
		
		for (int i=0; i<=7; i++)
		{
			for (int j=0; j<=7; j++)
			{
				tile[i][j] = old[7-i][j];
			}
		}
	}
	
	public void rotateClockwise()
	{
		int old[][] = new int[8][8];
		for (int i=0; i<=7; i++)
		{			
			for (int j=0; j<=7; j++)
			{
				old[i][j] = tile[i][j]; 
			}
		}
		
		for (int i=0; i<=7; i++)
		{
			for (int j=0; j<=7; j++)
			{
				tile[i][j] = old[j][7-i];
			}
		}
	}
}