package de.digisalt.tileEdit.view.components;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JPanel;

import de.digisalt.tileEdit.controller.Commands;
import de.digisalt.tileEdit.controller.Controller;

@SuppressWarnings("serial")
public class ColorAdjustmentBar extends JPanel {
	private int minimum = 0;
	private int maximum = 31;
	private int value = 20;
	private String colorName = null;
	@SuppressWarnings("unused")
	private String displayText = null;
	public static final int COLOR_RED = 1;
	public static final int COLOR_GREEN = 2;
	public static final int COLOR_BLUE = 3;
	
	public ColorAdjustmentBar(int colorType)
	{
		if (colorType == COLOR_RED)
		{
			colorName = "red";
		}
		else if (colorType == COLOR_GREEN)
		{
			colorName = "green";
		}
		else if (colorType == COLOR_BLUE)
		{
			colorName = "blue";
		}
		else
		{
			colorName = "black";
		}
		this.addMouseListener(new MouseListener()
		{
			public void mouseClicked(MouseEvent arg0) 
			{
				changeColor(arg0.getX());	
				Controller.getInstance().modelToView(Commands.m2vRepaintAllObjectsOnTileEditor, null);
			}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) 
			{
				changeColor(arg0.getX());
			}
			public void mouseReleased(MouseEvent arg0) 
			{
				changeColor(arg0.getX());
				Controller.getInstance().modelToView(Commands.m2vRepaintAllObjectsOnTileEditor, null);
			}
		});
		this.addMouseMotionListener(new MouseMotionListener()
		{
			public void mouseDragged(MouseEvent arg0) 
			{
				changeColor(arg0.getX());
				Controller.getInstance().modelToView(Commands.m2vRepaintColorAdjustersOnly, null);
				Controller.getInstance().modelToView(Commands.m2vRepaintPaletteOnly, null);
			}
			public void mouseMoved(MouseEvent arg0) {}
		});
	}
	
	public int getValue()
	{
		return this.value;
	}
	
	public void setValue(int value)
	{
		if (value < minimum || value > maximum)
		{}
		else
		{
			this.value = value;			
		}
	}
	
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);

		//draw Progressbar
		if (colorName.equals("red"))
		{
			g.setColor(new Color(255,127,127));
		}
		else if (colorName.equals("green"))
		{
			g.setColor(new Color(127,255,127));
		}
		else if (colorName.equals("blue"))
		{
			g.setColor(new Color(127,127,255));
		}
		else
		{
			g.setColor(new Color(0,0,0));
		}
		g.fillRect(27, 0, 4*getValue(), 20);
		//draw Progressbar again
		if (colorName.equals("red"))
		{
			g.setColor(new Color(255,63,63));
		}
		else if (colorName.equals("green"))
		{
			g.setColor(new Color(63,255,63));
		}
		else if (colorName.equals("blue"))
		{
			g.setColor(new Color(63,63,255));
		}
		else
		{
			g.setColor(new Color(0,0,0));
		}
		g.fillRect(27, 3, 4*getValue()-1, 17);
		//draw Progressbar again
		if (colorName.equals("red"))
		{
			g.setColor(new Color(255,31,31));
		}
		else if (colorName.equals("green"))
		{
			g.setColor(new Color(31,255,31));
		}
		else if (colorName.equals("blue"))
		{
			g.setColor(new Color(31,31,255));
		}
		else
		{
			g.setColor(new Color(0,0,0));
		}
		g.fillRect(27, 7, 4*getValue()-2, 13);
		//draw Progressbar again
		if (colorName.equals("red"))
		{
			g.setColor(new Color(255,0,0));
		}
		else if (colorName.equals("green"))
		{
			g.setColor(new Color(0,255,0));
		}
		else if (colorName.equals("blue"))
		{
			g.setColor(new Color(0,0,255));
		}
		else
		{
			g.setColor(new Color(0,0,0));
		}
		g.fillRect(27, 15, 4*getValue()-4, 5);
		g.setColor(new Color(0,0,0));
		g.drawRect(26, 0, 125, 20);
		//143
		//draw Text ontop
		displayText = colorName + ": " + this.getValue();
		g.setFont(new Font(Font.MONOSPACED, Font.BOLD, 12));
		if (colorName.equals("red"))
		{
			g.drawString("R", 16, 15);			
		}
		else if (colorName.equals("green"))
		{
			g.drawString("G", 16, 15);
		}
		else if (colorName.equals("blue"))
		{
			g.drawString("B", 16, 15);
		}
		g.drawString(""+getValue(), 80, 15);
	}
	
	private void changeColor(int x)
	{
		if (x >= 16 && x <= 26)
		{
			value = 0;//28-150
			repaint();
			setColorParameter(colorName, value);
		}
		else if (x > 26)
		{
			x = x - 24;
			value = Math.round(x / 4);
			if (value > 31) value = 31;
			repaint();
			setColorParameter(colorName, value);
		}
	}
	
	private void setColorParameter(String colorType, int val)
	{
		if (colorType.equals("red"))
		{
			Controller.getInstance().viewToModel(Commands.v2mSetCurrentRedValue, new Object[] {new Integer(val)});			
		}
		else if (colorType.equals("green"))
		{
			Controller.getInstance().viewToModel(Commands.v2mSetCurrentGreenValue, new Object[] {new Integer(val)});
		}
		else if (colorType.equals("blue"))
		{
			Controller.getInstance().viewToModel(Commands.v2mSetCurrentBlueValue, new Object[] {new Integer(val)});
		}
	}
}
