package de.digisalt.tileEdit.view.components;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;

import javax.swing.JPanel;

@SuppressWarnings("serial")
public class StatusBar extends JPanel {
	private String msg = "Digisalt Java Swing Statusbar";
	public static final int STATUS_NONE = 0;
	public static final int STATUS_WARN = 1;
	public static final int STATUS_ERROR = 2;
	public static final int STATUS_OK = 3;
	private int x = 8;
	private int y = 5;
	private int value = STATUS_NONE;
	
	public StatusBar()
	{
		super();
		this.setPreferredSize(new Dimension(400,22));
	}
	
	public void setMessage(int MessageType, String msg)
	{
		if (MessageType < 1 || MessageType > 3)
		{
			this.value = STATUS_NONE;
		}
		else
		{
			this.value = MessageType;
		}
		this.msg = msg;
	}
	
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		
		//paint Background
		paintBackground(g);
		
		//paint icon
		if (value == STATUS_ERROR) paintError(g);
		else if (value == STATUS_OK) paintOk(g);
		else if (value == STATUS_WARN) paintWarn(g);
		
		//paint String
		g.setColor(new Color(0,0,0));
		g.setFont(new Font(Font.MONOSPACED, Font.BOLD, 12));
		String printMsg = "";
		if (value == STATUS_ERROR) printMsg = "Error: " + msg;
		else if (value == STATUS_WARN) printMsg = "Warning: " + msg;
		else if (value == STATUS_OK) printMsg = "Ok: " + msg;
		else printMsg = msg;
		
		//if no icon set --> print text more on the left
		if (value < 1 || value > 3)
		{
			g.drawString(printMsg, 15, 17);			
		}
		else
		{
			g.drawString(printMsg, 30, 17);
		}
	}
	
	private void paintBackground(Graphics g)
	{
		//paint white fading background rectangle
		g.setColor(new Color(255,255,255));
		g.fillRect(0, 0, 200, 22);
		g.setColor(new Color(250,250,250));
		g.fillRect(200, 0, 20, 22);
		g.setColor(new Color(245,245,245));
		g.fillRect(220, 0, 20, 22);
		g.setColor(new Color(240,240,240));
		g.fillRect(240, 0, 20, 22);
		g.setColor(new Color(238,238,238));
		g.fillRect(260, 0, 20, 22);
		//paint dark grey fading border line
		g.setColor(new Color(190,190,190));
		g.fillRect(0, 0, 180, 1);
		g.setColor(new Color(200,200,200));
		g.fillRect(180, 0, 20, 1);
		g.setColor(new Color(210,210,210));
		g.fillRect(200, 0, 20, 1);
		g.setColor(new Color(220,220,220));
		g.fillRect(220, 0, 20, 1);
		g.setColor(new Color(230,230,230));
		g.fillRect(240, 0, 20, 1);
		g.setColor(new Color(238,238,238));
		g.fillRect(260, 0, 20, 1);
	}

	private void paintError(Graphics g)
	{
		//outline
		g.setColor(new Color(0,0,0));
		g.drawLine(x+0, y+4, x+4, y+0);
		g.drawLine(x+4, y+0, x+7, y+3);
		g.drawLine(x+8, y+3, x+11, y+0);
		g.drawLine(x+11, y+0, x+15, y+4);
		g.drawLine(x+15, y+4, x+12, y+7);
		g.drawLine(x+12, y+8, x+15, y+11);
		g.drawLine(x+15, y+11, x+11, y+15);
		g.drawLine(x+11, y+15, x+8, y+12);
		g.drawLine(x+7, y+12, x+4, y+15);
		g.drawLine(x+4, y+15, x+0, y+11);
		g.drawLine(x+0, y+11, x+3, y+8);
		g.drawLine(x+3, y+7, x+0, y+4);
		//dark red parts
		g.setColor(new Color(168,0,0));
		g.drawLine(x+1, y+4, x+3, y+2);
		g.drawLine(x+1, y+4, x+4, y+7);
		g.drawLine(x+2, y+4, x+4, y+6);
		g.drawLine(x+1, y+11, x+3, y+9);
		g.drawLine(x+1, y+11, x+4, y+14);
		g.drawLine(x+4, y+14, x+6, y+12);
		g.drawLine(x+2, y+11, x+4, y+13);
		g.drawLine(x+8, y+11, x+11, y+14);
		g.drawLine(x+11, y+14, x+13, y+12);
		g.drawLine(x+9, y+11, x+11, y+13);
		//medium red parts
		g.setColor(new Color(224,0,0));
		g.drawLine(x+4, y+1, x+3, y+4);
		g.drawLine(x+3, y+4, x+5, y+6);
		g.drawLine(x+14, y+11, x+11, y+12);
		g.drawLine(x+11, y+12, x+9, y+10);
		g.drawLine(x+3, y+10, x+3, y+11);
		g.drawLine(x+4, y+8, x+4, y+12);
		g.drawLine(x+5, y+6, x+5, y+12);
		g.drawLine(x+6, y+9, x+6, y+11);
		g.drawLine(x+7, y+10, x+7, y+11);
		g.drawLine(x+8, y+10, x+8, y+10);
		//light red parts
		g.setColor(new Color(255,0,0));
		g.drawLine(x+4, y+3, x+4, y+4);
		g.drawLine(x+5, y+2, x+5, y+5);
		g.drawLine(x+6, y+3, x+6, y+8);
		g.drawLine(x+7, y+4, x+7, y+9);
		g.drawLine(x+8, y+4, x+8, y+9);
		g.drawLine(x+9, y+3, x+9, y+9);
		g.drawLine(x+10, y+2, x+10, y+10);
		g.drawLine(x+11, y+1, x+11, y+11);
		g.drawLine(x+12, y+2, x+12, y+6);
		g.drawLine(x+13, y+3, x+13, y+5);
		g.drawLine(x+14, y+4, x+14, y+4);
		g.drawLine(x+12, y+9, x+12, y+11);
		g.drawLine(x+13, y+10, x+13, y+10);
	}
	
	private void paintOk(Graphics g)
	{
		//outline
		g.setColor(new Color(0,0,0));
		g.drawLine(x+0, y+7, x+2, y+5);
		g.drawLine(x+2, y+5, x+4, y+7);
		g.drawLine(x+4, y+7, x+11, y+0);
		g.drawLine(x+11, y+0, x+15, y+4);
		g.drawLine(x+0, y+7, x+6, y+13);
		g.drawLine(x+6, y+13, x+15, y+4);
		//parts with darkest green
		g.setColor(new Color(0,130,0));
		g.drawLine(x+1, y+7, x+6, y+12);
		g.drawLine(x+5, y+10, x+6, y+11);
		g.drawLine(x+7, y+11, x+7, y+11);
		//parts with medium green
		g.setColor(new Color(0,173,0));
		g.drawLine(x+2, y+7, x+4, y+9);
		g.drawLine(x+5, y+9, x+6, y+9);
		g.drawLine(x+6, y+10, x+8, y+10);
		//parts with normal green
		g.setColor(new Color(0,216,0));
		g.drawLine(x+2, y+6, x+4, y+8);
		g.drawLine(x+4, y+8, x+5, y+7);
		g.drawLine(x+4, y+8, x+8, y+8);
		g.drawLine(x+7, y+9, x+9, y+9);
		//parts with light green
		g.setColor(new Color(0,255,0));
		g.drawLine(x+9, y+8, x+10, y+8);
		g.drawLine(x+6, y+7, x+11, y+7);
		g.drawLine(x+6, y+6, x+12, y+6);
		g.drawLine(x+7, y+5, x+13, y+5);
		g.drawLine(x+8, y+4, x+14, y+4);
		g.drawLine(x+9, y+3, x+13, y+3);
		g.drawLine(x+10, y+2, x+12, y+2);
		g.drawLine(x+11, y+1, x+11, y+1);
	}

	private void paintWarn(Graphics g)
	{
		//outline
		g.setColor(new Color(0,0,0));
		g.drawLine(x+6, y+0, x+9, y+0);
		g.drawLine(x+5, y+1, x+0, y+12);
		g.drawLine(x+10, y+1, x+15, y+12);
		g.drawLine(x+0, y+13, x+1, y+14);
		g.drawLine(x+15, y+13, x+14, y+14);
		g.drawLine(x+2, y+14 ,x+13, y+14);
		//black !
		g.drawRect(x+7, y+3, 1, 5);
		g.drawRect(x+6, y+4, 3, 1);
		g.drawRect(x+7, y+11, 1, 1);
		//parts with darkest yellow
		g.setColor(new Color(131,131,0));
		g.drawLine(x+6, y+6, x+6, y+6);
		g.drawLine(x+9, y+6, x+9, y+6);
		g.drawLine(x+1, y+11, x+1, y+13);
		g.drawLine(x+2, y+12, x+2, y+13);
		g.drawLine(x+3, y+13, x+4, y+13);
		g.drawLine(x+13, y+13, x+14, y+13);
		g.drawLine(x+14, y+11, x+14, y+12);
		//parts with medium yellow
		g.setColor(new Color(195,195,0));
		g.drawLine(x+6, y+3, x+6, y+3);
		g.drawLine(x+9, y+3, x+9, y+3);
		g.drawLine(x+6, y+7, x+6, y+7);
		g.drawLine(x+9, y+7, x+9, y+7);
		g.drawLine(x+2, y+10, x+2, y+11);
		g.drawLine(x+3, y+11, x+3, y+12);
		g.drawLine(x+4, y+12, x+5, y+12);
		g.drawLine(x+6, y+11, x+6, y+12);
		g.drawLine(x+5, y+13, x+12, y+13);
		g.drawLine(x+9, y+11, x+9, y+12);
		g.drawLine(x+7, y+10, x+8, y+10);
		g.drawLine(x+12, y+12, x+13, y+12);
		g.drawLine(x+13, y+10, x+13, y+11);
		//parts with light yellow
		g.setColor(new Color(255,255,0));
		g.drawLine(x+6, y+1, x+9, y+1);
		g.drawLine(x+6, y+2, x+9, y+2);
		g.drawLine(x+3, y+7, x+3, y+10);
		g.drawLine(x+4, y+5, x+4, y+11);
		g.drawLine(x+5, y+3, x+5, y+11);
		g.drawLine(x+6, y+8, x+6, y+10);
		g.drawLine(x+9, y+8, x+9, y+10);
		g.drawLine(x+10, y+3, x+10, y+12);
		g.drawLine(x+11, y+5, x+11, y+12);
		g.drawLine(x+12, y+7, x+12, y+11);
		g.drawLine(x+2, y+9, x+13, y+9);
	}
}
