package de.digisalt.tileEdit.view.components;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JPanel;

import de.digisalt.tileEdit.controller.Commands;
import de.digisalt.tileEdit.controller.Controller;
import de.digisalt.tileEdit.model.CurrentData;
import de.digisalt.tileEdit.model.dataContainers.AbstractPaletteContainer;
import de.digisalt.tileEdit.model.dataContainers.TileContainer;

@SuppressWarnings("serial")
public class TilePanel16x extends JPanel 
{
	private boolean leftButton = true;
	private AbstractPaletteContainer pal = null;
	private boolean containsData = false;
	private boolean gridEnabled = false;
	private TileContainer data = null;
	
	public TilePanel16x(AbstractPaletteContainer newPal)
	{
		data = new TileContainer(true);
		pal = newPal;
		this.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		handleMouseInteraction();
	}
	
	public TilePanel16x(AbstractPaletteContainer newPal, TileContainer t)
	{
		data = t;
		containsData = true;
		pal = newPal;
		this.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		handleMouseInteraction();
	}
	
	public TileContainer getData()
	{
		return data;
	}
	
	public void setData(TileContainer t)
	{
		data = t;
		containsData = true;
		pal = CurrentData.getInstance().getCurrentPalette();
	}
	
	public void removeData()
	{
		data = new TileContainer(true);
		containsData = false;
		pal = CurrentData.getInstance().getCurrentPalette();
	}
	
	public void paintComponent(Graphics g) 
	{
        super.paintComponent(g);       

        // Draw Pixels
        for (int i = 0; i<=7; i++)
        {
        	for (int j = 0; j<=7; j++)
        	{
        		//getCurrentPixelColor
        		try {
					g.setColor(pal.getColor(data.getColorAt(i, j)));
				} catch (Exception e) {
					Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Error setting color to draw."});
				}
        		g.fillRect(i*16,j*16,16,16);
        	}
        }  
        //draw Gridlines if wanted
        if (gridEnabled)
        {
        	g.setColor(new Color(0,0,0));
        	g.drawRect(0, 0, 31, 31);
        }
        
    }  
	
	public boolean containsData()
	{
		return containsData;
	}
	
	public void setGridEnabled(boolean val)
	{
		gridEnabled = val;
	}
	
	public boolean getGridEnabled()
	{
		return gridEnabled;
	}
	
	private void handleMouseInteraction()
	{
		this.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) 
			{
				int[] bla = getCurrentPixel(arg0.getX(), arg0.getY());
				if (bla[0] >= 0 && bla[1] >= 0 && bla[0] <= 7 && bla[1] <= 7)
				{
					if (arg0.getButton() == MouseEvent.BUTTON1)
					{
						leftButton = true;
						setPixelNumber(bla[0], bla[1], CurrentData.getInstance().getCurrentLeftColorNumber());						
						//Controller.getInstance().modelToView(Commands.m2vRepaintAllObjectsOnTileEditor, null);
					}
					else if (arg0.getButton() == MouseEvent.BUTTON3)
					{
						leftButton = false;
						setPixelNumber(bla[0], bla[1], CurrentData.getInstance().getCurrentRightColorNumber());
						//Controller.getInstance().modelToView(Commands.m2vRepaintAllObjectsOnTileEditor, null);
					}
				}
			}
			public void mouseReleased(MouseEvent arg0) {
				int[] bla = getCurrentPixel(arg0.getX(), arg0.getY());
				if (bla[0] >= 0 && bla[1] >= 0 && bla[0] <= 7 && bla[1] <= 7)
				{
					if (arg0.getButton() == MouseEvent.BUTTON1)
					{
						leftButton = true;
						setPixelNumber(bla[0], bla[1], CurrentData.getInstance().getCurrentLeftColorNumber());						
					}
					else if (arg0.getButton() == MouseEvent.BUTTON3)
					{
						leftButton = false;
						setPixelNumber(bla[0], bla[1], CurrentData.getInstance().getCurrentRightColorNumber());
					}					
				}
				Controller.getInstance().modelToView(Commands.m2vRepaintAllObjectsOnTileEditor, null);
			}
		});
		this.addMouseMotionListener(new MouseMotionListener() {
			public void mouseDragged(MouseEvent arg0) 
			{
				int[] bla = getCurrentPixel(arg0.getX(), arg0.getY());
				if (bla[0] >= 0 && bla[1] >= 0 && bla[0] <= 7 && bla[1] <= 7)
				{
					if (leftButton)
					{
						setPixelNumber(bla[0], bla[1], CurrentData.getInstance().getCurrentLeftColorNumber());
						
						Controller.getInstance().modelToView(Commands.m2vRepaintBigTileOnly, null);						
					}
					else if (!leftButton)
					{
						setPixelNumber(bla[0], bla[1], CurrentData.getInstance().getCurrentRightColorNumber());
						Controller.getInstance().modelToView(Commands.m2vRepaintBigTileOnly, null);						
					}
				}
			}
			public void mouseMoved(MouseEvent arg0) {}
		});
	}
	
	private int[] getCurrentPixel(int x, int y)
	{
		int row = -1;
		int col = -1;
		col = Math.round(x / 16);
		row = Math.round(y / 16);
		
		if (row < 0 || col < 0 || row > 8 || col > 8) return new int[] {-1, -1};
		//System.out.println(row * 8 + col +1);
		return new int[] {col, row};
	}
	
	private void setPixelNumber(int x, int y, int colorIndex)
	{
		try {
			data.setColorAt(x, y, colorIndex);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
