package de.digisalt.tileEdit.view.components;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.TransferHandler;

import de.digisalt.tileEdit.controller.Commands;
import de.digisalt.tileEdit.controller.Controller;
import de.digisalt.tileEdit.model.CurrentData;
import de.digisalt.tileEdit.model.dataContainers.AbstractPaletteContainer;
import de.digisalt.tileEdit.model.dataContainers.TileContainer;

@SuppressWarnings("serial")
public class TilePanel2x extends JPanel 
{
	private AbstractPaletteContainer pal = null;
	private boolean containsData = false;
	private boolean gridEnabled = false;
	private TileContainer data = null;
	private boolean isFilled = true;
	
	public TilePanel2x(AbstractPaletteContainer newPal)
	{
		data = new TileContainer(true);
		pal = newPal;
		this.setTransferHandler(new TransferHandler("copy"));
		this.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) 
			{
				Controller.getInstance().modelToView(Commands.m2vRepaintArrangerOnly, null);
				Controller.getInstance().modelToView(Commands.m2vRepaintTileSourceOnly, null);
				JComponent c = (JComponent)e.getSource();
				TransferHandler th = c.getTransferHandler();
				th.exportAsDrag(c, e, TransferHandler.COPY);
		    }
		});
	}
	
	public TilePanel2x(AbstractPaletteContainer newPal, TileContainer t)
	{
		data = t;
		containsData = true;
		pal = newPal;
		this.setTransferHandler(new TransferHandler("copy"));
		this.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) 
			{
				Controller.getInstance().modelToView(Commands.m2vRepaintArrangerOnly, null);
				Controller.getInstance().modelToView(Commands.m2vRepaintTileSourceOnly, null);
				JComponent c = (JComponent)e.getSource();
				TransferHandler th = c.getTransferHandler();
				th.exportAsDrag(c, e, TransferHandler.COPY);
		    }
		});
	}
	
	public TileContainer getData()
	{
		return data;
	}
	
	public void setData(TileContainer t)
	{
		data = t;
		containsData = true;
		pal = CurrentData.getInstance().getCurrentPalette();
	}
	
	public void removeData()
	{
		data = new TileContainer(true);
		containsData = false;
		pal = CurrentData.getInstance().getCurrentPalette();
	}
	
	public TileContainer getCopy()
	{
		if (!isFilled) return null;
		return data;
	}
	
	public void setCopy(TileContainer newData)
	{
		if (isFilled)
		{
			data = newData;			
			repaint();
		}
	}
	
	public void paintComponent(Graphics g) 
	{
        super.paintComponent(g);       

        // Draw Pixels
        for (int i = 0; i<=7; i++)
        {
        	for (int j = 0; j<=7; j++)
        	{
        		//getCurrentPixelColor
        		try {
					g.setColor(pal.getColor(data.getColorAt(i, j)));
				} catch (Exception e) {
					Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Error setting color to draw."});
				}
        		g.fillRect(i*2,j*2,2,2);
        	}
        }  
        //draw Gridlines if wanted
        if (gridEnabled)
        {
        	g.setColor(new Color(0,0,0));
        	g.drawRect(0, 0, 15, 15);
        }
        
    }  
	
	public boolean containsData()
	{
		return containsData;
	}
	
	public void setGridEnabled(boolean val)
	{
		gridEnabled = val;
	}
	
	public boolean getGridEnabled()
	{
		return gridEnabled;
	}
	
	public boolean getIsFilled()
	{
		return isFilled;
	}
	
	public void setIsFilled(boolean val)
	{
		isFilled = val;
	}
}
