package de.digisalt.tileEdit.view.windows;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;
import javax.swing.TransferHandler;

import de.digisalt.tileEdit.controller.Commands;
import de.digisalt.tileEdit.controller.Controller;
import de.digisalt.tileEdit.model.CurrentData;
import de.digisalt.tileEdit.model.dataContainers.TileContainer;
import de.digisalt.tileEdit.model.fileFilter.SnesPaletteFilter;
import de.digisalt.tileEdit.model.fileFilter.SnesTileFilter;
import de.digisalt.tileEdit.view.components.ColorAdjustmentBar;
import de.digisalt.tileEdit.view.components.ColorSelectionPanel;
import de.digisalt.tileEdit.view.components.NewFileDialog;
import de.digisalt.tileEdit.view.components.PalettePanel;
import de.digisalt.tileEdit.view.components.StatusBar;
import de.digisalt.tileEdit.view.components.TilePanel16x;
import de.digisalt.tileEdit.view.components.TilePanel2x;
import de.digisalt.tileEdit.view.components.TilePanel4x;
import de.digisalt.tileEdit.view.helpers.IconFactory;
import de.digisalt.tileEdit.view.layout.HIGConstraints;
import de.digisalt.tileEdit.view.layout.HIGLayout;

@SuppressWarnings("serial")
public class MainWindow extends JFrame {
	private static MainWindow mainWin = null;
	private TilePanel4x[][] tileMatrix = new TilePanel4x[16][16];
	private TilePanel2x[][] arraMatrix = new TilePanel2x[24][16];
	private TilePanel16x editTile = null;
	private ColorSelectionPanel colSelect = null;
	private PalettePanel palPanel = null;
	private ColorAdjustmentBar red = null;
	private ColorAdjustmentBar green = null;
	private ColorAdjustmentBar blue = null;
	private JLabel tiles2 = null;
	private JPanel content = null;
	private JRadioButtonMenuItem col1, col2, col3, col4, col5, col6, col7, col8, col9, col10, col11, col12, col13, col14, col15, col16;
	private StatusBar statusBar = null;

	private MainWindow()
	{
		this.setSize(970, 660);
		
		this.setTitle("Digisalt SNES TileEditor v0.1");
		for (int i=0; i<=15; i++)
		{
			for (int j=0; j<=15; j++)
			{
				tileMatrix[i][j] = new TilePanel4x(CurrentData.getInstance().getCurrentPalette(), new TileContainer(false));
			}
		}
		
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		initialize();
	}

	
	public static MainWindow getInstance() {
		if (mainWin == null)
		{
			mainWin = new MainWindow();
		}
		return mainWin;
	}
	
	private void initialize()
	{
		JLayeredPane main = new JLayeredPane();
		main.setLayout(new BorderLayout()); 
		JMenuBar menu = new JMenuBar();
		main.add(menu, BorderLayout.NORTH);
		this.add(main);
		statusBar = new StatusBar();
		main.add(statusBar, BorderLayout.SOUTH);
		
		//all menuitems in menubar
		JMenu file = new JMenu("File");
		file.setMnemonic(KeyEvent.VK_F);
		JMenu edit = new JMenu("Edit");
		edit.setMnemonic(KeyEvent.VK_E);
		JMenu view = new JMenu("View");
		view.setMnemonic(KeyEvent.VK_V);
		JMenu palette = new JMenu("Palette");
		palette.setMnemonic(KeyEvent.VK_P);
		JMenu help = new JMenu("Help");
		help.setMnemonic(KeyEvent.VK_H);
		menu.add(file);
		menu.add(edit);
		menu.add(view);
		menu.add(palette);
		menu.add(help);
		
		//file menu
		JMenuItem fileOpen = new JMenuItem("Open");
		fileOpen.setMnemonic(KeyEvent.VK_O);
		fileOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
		fileOpen.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) {
				JFileChooser fc = new JFileChooser();
				fc.setFileFilter(new SnesTileFilter());
				int returnVal = fc.showOpenDialog(MainWindow.this);
				if (returnVal == JFileChooser.APPROVE_OPTION)
				{
					//TODO: old file modified? --> save
					Controller.getInstance().viewToModel(Commands.v2mLoadGraphicIntoEditor, new Object[]{fc.getSelectedFile().getAbsolutePath()});
				}
			}
		});
		JMenuItem fileNew = new JMenuItem("New");
		fileNew.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				//TODO: old file modified? --> save
				NewFileDialog newFile = NewFileDialog.getInstance(MainWindow.getInstance());
				newFile.setVisible(true);
			}
		});
		fileNew.setMnemonic(KeyEvent.VK_N);
		fileNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		JMenuItem fileClose = new JMenuItem("Close");
		fileClose.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				//TODO: old file modified? --> save
				Controller.getInstance().viewToModel(Commands.v2mCloseFile, null);
			}
		});
		fileClose.setMnemonic(KeyEvent.VK_C);
		JMenuItem fileSave = new JMenuItem("Save");
		fileSave.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				//if there's no tile to save, ignore click
				if (CurrentData.getInstance().getMaxTiles() > 0)
				{
					//tiles available --> filename already set?
					if (!CurrentData.getInstance().getCurrentFilePath().equals(""))
					{
						//filename already existant --> save without dialog
						Controller.getInstance().viewToModel(Commands.v2mSaveGraphicToFile, new Object[] {CurrentData.getInstance().getCurrentFilePath()});
					}
					else
					{
						//TODO: no filename present --> open save as dialog
					}
				}
			}
		});
		fileSave.setMnemonic(KeyEvent.VK_S);
		fileSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		JMenuItem fileSaveAs = new JMenuItem("Save As...");
		fileSaveAs.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				JFileChooser fc = new JFileChooser();
				fc.setFileFilter(new SnesTileFilter());
				int returnVal = fc.showSaveDialog(MainWindow.this);
				if (returnVal == JFileChooser.APPROVE_OPTION)
				{
					Controller.getInstance().viewToModel(Commands.v2mSaveGraphicToFile, new Object[]{fc.getSelectedFile().getAbsolutePath()});
				}
			}
		});
		fileSaveAs.setMnemonic(KeyEvent.VK_A);
		JMenuItem fileExit = new JMenuItem("Exit");
		fileExit.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				//TODO: old file modified? --> save
				System.exit(0);
			}
		});
		fileExit.setMnemonic(KeyEvent.VK_X);
		file.add(fileOpen);
		file.add(fileNew);
		file.add(fileClose);
		file.add(fileSave);
		file.add(fileSaveAs);
		file.add(fileExit);
		
		//edit menu
		JMenuItem editImport = new JMenuItem("Import Bitmap...");
		editImport.setMnemonic(KeyEvent.VK_I);
		JMenuItem editExport = new JMenuItem("Export Bitmap...");
		editExport.setMnemonic(KeyEvent.VK_E);
		JMenuItem editClear = new JMenuItem("Clear Arranger");
		editClear.setMnemonic(KeyEvent.VK_C);
		editClear.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				clearArranger();
			}
		});
		edit.add(editImport);
		edit.add(editExport);
		edit.addSeparator();
		edit.add(editClear);
		
		//view menu
		JMenu cols = new JMenu("Columns");
		cols.setMnemonic(KeyEvent.VK_C);
		ButtonGroup colGroup = new ButtonGroup();
		col1 = new JRadioButtonMenuItem("1x");
		col1.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(1)});
			}
		});
		col2 = new JRadioButtonMenuItem("2x");
		col2.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(2)});
			}
		});
		col3 = new JRadioButtonMenuItem("3x");
		col3.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(3)});
			}
		});
		col4 = new JRadioButtonMenuItem("4x");
		col4.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(4)});
			}
		});
		col5 = new JRadioButtonMenuItem("5x");
		col5.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(5)});
			}
		});
		col6 = new JRadioButtonMenuItem("6x");
		col6.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(6)});
			}
		});
		col7 = new JRadioButtonMenuItem("7x");
		col7.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(7)});
			}
		});
		col8 = new JRadioButtonMenuItem("8x");
		col8.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(8)});
			}
		});
		col9 = new JRadioButtonMenuItem("9x");
		col9.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(9)});
			}
		});
		col10 = new JRadioButtonMenuItem("10x");
		col10.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(10)});
			}
		});
		col11 = new JRadioButtonMenuItem("11x");
		col11.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(11)});
			}
		});
		col12 = new JRadioButtonMenuItem("12x");
		col12.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(12)});
			}
		});
		col13 = new JRadioButtonMenuItem("13x");
		col13.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(13)});
			}
		});
		col14 = new JRadioButtonMenuItem("14x");
		col14.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(14)});
			}
		});
		col15 = new JRadioButtonMenuItem("15x");
		col15.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(15)});
			}
		});
		col16 = new JRadioButtonMenuItem("16x");
		col16.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mSetNumberOfColumns, new Object[]{new Integer(16)});
			}
		});
		col16.setSelected(true);
		colGroup.add(col1);
		colGroup.add(col2);
		colGroup.add(col3);
		colGroup.add(col4);
		colGroup.add(col5);
		colGroup.add(col6);
		colGroup.add(col7);
		colGroup.add(col8);
		colGroup.add(col9);
		colGroup.add(col10);
		colGroup.add(col11);
		colGroup.add(col12);
		colGroup.add(col13);
		colGroup.add(col14);
		colGroup.add(col15);
		colGroup.add(col16);
		cols.add(col1);
		cols.add(col2);
		cols.add(col3);
		cols.add(col4);
		cols.add(col5);
		cols.add(col6);
		cols.add(col7);
		cols.add(col8);
		cols.add(col9);
		cols.add(col10);
		cols.add(col11);
		cols.add(col12);
		cols.add(col13);
		cols.add(col14);
		cols.add(col15);
		cols.add(col16);
		final JCheckBoxMenuItem viewGrid = new JCheckBoxMenuItem("Gridlines");
		viewGrid.setMnemonic(KeyEvent.VK_G);
		viewGrid.setState(true);
		viewGrid.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				if (viewGrid.isSelected())
				{
					CurrentData.getInstance().setGridEnabled(true);
				}
				else
				{
					CurrentData.getInstance().setGridEnabled(false);
				}
				Controller.getInstance().modelToView(Commands.m2vUpdateTiles, null);
			}
		});
		view.add(cols);
		view.add(viewGrid);
		
		//palette menu
		JMenuItem palLoad = new JMenuItem("Load...");
		palLoad.setMnemonic(KeyEvent.VK_L);
		palLoad.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) {
				JFileChooser fc = new JFileChooser();
				fc.setFileFilter(new SnesPaletteFilter());
				int returnVal = fc.showOpenDialog(MainWindow.this);
				if (returnVal == JFileChooser.APPROVE_OPTION)
				{
					//TODO: old file modified? --> save
					Controller.getInstance().viewToModel(Commands.v2mLoadPaletteIntoEditor, new Object[]{fc.getSelectedFile().getAbsolutePath()});
				}
			}
		});
		JMenuItem palSave = new JMenuItem("Save");
		palSave.setMnemonic(KeyEvent.VK_S);
		JMenuItem palSaveAs = new JMenuItem("Save As...");
		palSaveAs.setMnemonic(KeyEvent.VK_A);
		palette.add(palLoad);
		palette.add(palSave);
		palette.add(palSaveAs);
		
		//help menu
		JMenuItem helpContents = new JMenuItem("Contents");
		helpContents.setMnemonic(KeyEvent.VK_C);
		JMenuItem helpAbout = new JMenuItem("About...");
		helpAbout.setMnemonic(KeyEvent.VK_A);
		help.add(helpContents);
		help.add(helpAbout);
		
		//base panels on body
		int hh = 24;
		JLayeredPane body = new JLayeredPane();
		main.add(body, BorderLayout.CENTER);
		int w1[] = {12, 538, 12, 162, 224, 12};
		int h1[] = {12, 282, 12, 154, 12, 126, 12};
		body.setLayout(new HIGLayout(w1, h1));
		HIGConstraints c = new HIGConstraints();
		
		//content panel
		content = new JPanel();
		//content.setBackground(new Color(255,0,0));
		body.add(content, c.rcwh(2, 2, 1, 5));
		int w2[] = {514, hh};
		int h2[] = {hh, hh, hh, 418, hh, hh, hh, hh};
		content.setLayout(new HIGLayout(w2, h2));
		
		//tile arranger panel
		JPanel arranger = new JPanel();
		//arranger.setBackground(new Color(0,0,255));
		body.add(arranger, c.rcwh(2, 4, 2, 1));
		int w3[] = {386};
		int h3[] = {hh, 258};
		arranger.setLayout(new HIGLayout(w3, h3));
		
		//tile editor panel
		JPanel editor = new JPanel();
		//editor.setBackground(new Color(0,255,0));
		body.add(editor, c.rcwh(4, 4, 1, 1));
		int w4[] = {130, 32};
		int h4[] = {hh, 32, 32, 32, 32};
		editor.setLayout(new HIGLayout(w4, h4));
		
		//color palette panel
		JPanel pallette = new JPanel();
		//pallette.setBackground(new Color(255,255,255));
		body.add(pallette, c.rcwh(6, 4, 2, 1));
		int w5[] = {65, 169, 152};
		int h5[] = {hh, 12, 21,	5, 21, 5, 21, 17};
		pallette.setLayout(new HIGLayout(w5, h5));
		
		//content of tile overview area
		JLabel contentTitle = new JLabel("Available Tiles");
		content.add(contentTitle, c.rcwh(1, 1, 2, 1));
		JButton upup = new JButton(IconFactory.getImageIcon("upup.png"));
		upup.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mScrollUpUp, null);
			}
		});
		JButton up = new JButton(IconFactory.getImageIcon("up.png"));
		up.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mScrollUp, null);
			}
		});
		content.add(upup, c.rcwh(2, 2, 1, 1));
		content.add(up, c.rcwh(3, 2, 1, 1));
		JPanel scroller = new JPanel();
		scroller.setBackground(new Color(128, 128, 128));
		content.add(scroller, c.rcwh(4, 2, 1, 1));
		JButton down = new JButton(IconFactory.getImageIcon("down.png"));
		down.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mScrollDown, null);
			}
		});
		JButton downdown = new JButton(IconFactory.getImageIcon("downdown.png"));
		downdown.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				Controller.getInstance().viewToModel(Commands.v2mScrollDownDown, null);
			}
		});
		content.add(down, c.rcwh(5, 2, 1, 1));
		content.add(downdown, c.rcwh(6, 2, 1, 1));
		//tiles visible there
		JLabel tiles = new JLabel();
		int w6[] = {32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32};
		tiles.setLayout(new HIGLayout(w6, w6));
		tiles.setBorder(BorderFactory.createLineBorder(new Color(0,0,0)));
		for (int i=0; i<=15; i++)
		{
			for (int j=0; j<=15; j++)
			{
				final int k = i;
				final int l = j;
				
				tiles.add(tileMatrix[i][j], c.rcwh(i+1, j+1, 1, 1));
				tileMatrix[i][j].setTransferHandler(new TransferHandler("copy"));
				tileMatrix[i][j].addMouseListener(new MouseAdapter() {
					public void mousePressed(MouseEvent e) 
					{
						editTile.setData(tileMatrix[k][l].getData());
						Controller.getInstance().modelToView(Commands.m2vRepaintBigTileOnly, null);
						JComponent c = (JComponent)e.getSource();
						TransferHandler th = c.getTransferHandler();
						th.exportAsDrag(c, e, TransferHandler.COPY);
				    }
				});
			}
		}
		content.add(tiles, c.rcwh(2, 1, 1, 5));
		
		//content of arranger area
		JLabel arrangerTitle = new JLabel("Tile Arranger");
		arranger.add(arrangerTitle, c.rcwh(1, 1, 1, 1));
		tiles2 = new JLabel();
		int w7[] = {16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16};
		int h7[] = {16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16};
		tiles2.setLayout(new HIGLayout(w7, h7));
		tiles2.setBorder(BorderFactory.createLineBorder(new Color(0,0,0)));
		for (int i=0; i<=15; i++)
		{
			for (int j=0; j<=23; j++)
			{
				arraMatrix[j][i] = new TilePanel2x(CurrentData.getInstance().getCurrentPalette(), new TileContainer(false));
				tiles2.add(arraMatrix[j][i], c.rcwh(i+1, j+1, 1, 1));
			}
		}
		arranger.add(tiles2, c.rcwh(2, 1, 1, 1));
		
		//content of tileeditor area
		JLabel tileTitle = new JLabel("Tile Editor");
		editor.add(tileTitle, c.rcwh(1, 1, 2, 1));
		editTile = new TilePanel16x(CurrentData.getInstance().getCurrentPalette(), tileMatrix[0][0].getData());
		editTile.setBorder(BorderFactory.createLineBorder(new Color(0,0,0)));
		editor.add(editTile, c.rcwh(2, 1, 1, 4));
		JButton horiz = new JButton(IconFactory.getImageIcon("horizFlip.png"));
		horiz.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				editTile.getData().flipHorizontal();
				MainWindow.getInstance().repaint();
			}
		});
		editor.add(horiz, c.rcwh(2, 2, 1, 1));
		JButton vertic = new JButton(IconFactory.getImageIcon("verticFlip.png"));
		vertic.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				editTile.getData().flipVertical();
				MainWindow.getInstance().repaint();
			}
		});
		editor.add(vertic, c.rcwh(3, 2, 1, 1));
		JButton rotate = new JButton(IconFactory.getImageIcon("rotate.png"));
		rotate.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				editTile.getData().rotateClockwise();
				MainWindow.getInstance().repaint();
			}
		});
		editor.add(rotate, c.rcwh(4, 2, 1, 1));
		JButton undo = new JButton(IconFactory.getImageIcon("undo.png"));
		editor.add(undo, c.rcwh(5, 2, 1, 1));
		
		//content of palette editor area
		JLabel paletteTitle = new JLabel("Palette Editor");
		pallette.add(paletteTitle, c.rcwh(1, 1, 3, 1));
		colSelect = new ColorSelectionPanel();
		colSelect.setPalette(CurrentData.getInstance().getCurrentPalette());
		pallette.add(colSelect, c.rcwh(2, 1, 1, 7));
		palPanel = new PalettePanel();
		palPanel.setPalette(CurrentData.getInstance().getCurrentPalette());
		pallette.add(palPanel, c.rcwh(2, 2, 1, 7));
		red = new ColorAdjustmentBar(ColorAdjustmentBar.COLOR_RED);
		red.setValue(20);
		pallette.add(red, c.rcwh(3, 3, 1, 1));
		green = new ColorAdjustmentBar(ColorAdjustmentBar.COLOR_GREEN);
		red.setValue(20);
		pallette.add(green, c.rcwh(5, 3, 1, 1));
		blue = new ColorAdjustmentBar(ColorAdjustmentBar.COLOR_BLUE);
		red.setValue(20);
		pallette.add(blue, c.rcwh(7, 3, 1, 1));	
	}


	public void setNewSelectedLeftColor(int val) {
		try {
			this.colSelect.setLeftIndex(val);
			Color coll = CurrentData.getInstance().getCurrentPalette().getColor(val);
			red.setValue(coll.getRed()/8);
			green.setValue(coll.getGreen()/8);
			blue.setValue(coll.getBlue()/8);
			red.repaint();
			green.repaint();
			blue.repaint();
			colSelect.repaint();
		} catch (Exception e) {
			Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Error setting new selected left color"});
		}
	}


	public void setNewSelectedRightColor(int val) {
		try {
			this.colSelect.setRightIndex(val);
			colSelect.repaint();
		} catch (Exception e) {
			Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Error setting new selected right value"});
		}		
	}


	public void repaintBigTile() {
		editTile.repaint();
	}


	public void repaintArranger() {
		tiles2.repaint();
	}


	public void repaintAdjusters() {
		red.repaint();
		green.repaint();
		blue.repaint();
	}


	public void repaintPalette() {
		palPanel.repaint();
	}

	public void repaintTileSource() {
		content.repaint();		
	}
	
	public void showAlert(String string) {
		JOptionPane.showMessageDialog(this, string);
	}


	public void setNewLoadedTiles() {
		int maxNumOfTiles = CurrentData.getInstance().getMaxTiles();
		int cols = CurrentData.getInstance().getNumOfCols();
		int firstToDisplay = CurrentData.getInstance().getFirstDisplayedTile();
		//set visible columns with new tile content
		int currentTile = firstToDisplay;
		for (int i=0; i<16; i++)
		{
			for (int j=0; j<cols; j++)
			{
				//disable up to one complete column in front of the first tile
				if (currentTile < 0)
				{
					//tileMatrix[i][j].setData(new TileContainer(false));
					tileMatrix[i][j].setEnabled(false);
					tileMatrix[i][j].setVisible(false);
					currentTile++;
				}
				//paint all tiles to be displayed
				else if (currentTile >=0 && currentTile < maxNumOfTiles)
				{
					tileMatrix[i][j].setData(CurrentData.getInstance().getTileNumber(currentTile));
					tileMatrix[i][j].setEnabled(true);
					tileMatrix[i][j].setVisible(true);
					currentTile++;
				}
				//disable all tile after the last one
				else
				{
					//tileMatrix[i][j].setData(new TileContainer(false));
					tileMatrix[i][j].setEnabled(false);
					tileMatrix[i][j].setVisible(false);
				}
			}
		}
		//disable and set invisible the other tiles
		for (int i=0; i<16; i++)
		{
			for (int j=cols; j<16; j++)
			{
				tileMatrix[i][j].setData(new TileContainer(false));
				tileMatrix[i][j].setEnabled(false);
				tileMatrix[i][j].setVisible(false);
			}
		}
	}
	
	public void clearArranger()
	{
		for (int i=0; i<=15; i++)
		{
			for (int j=0; j<=23; j++)
			{
				arraMatrix[j][i].setData(new TileContainer(false));				
			}
		}
		repaintArranger();
	}


	public void setColSelection(int i) {
		if (i == 1) col1.setSelected(true);
		if (i == 2) col2.setSelected(true);
		if (i == 3) col3.setSelected(true);
		if (i == 4) col4.setSelected(true);
		if (i == 5) col5.setSelected(true);
		if (i == 6) col6.setSelected(true);
		if (i == 7) col7.setSelected(true);
		if (i == 8) col8.setSelected(true);
		if (i == 9) col9.setSelected(true);
		if (i == 10) col10.setSelected(true);
		if (i == 11) col11.setSelected(true);
		if (i == 12) col12.setSelected(true);
		if (i == 13) col13.setSelected(true);
		if (i == 14) col14.setSelected(true);
		if (i == 15) col15.setSelected(true);
		if (i == 16) col16.setSelected(true);
	}
	
	public void setStatusMessage(int type, String msg)
	{
		statusBar.setMessage(type, msg);
		statusBar.repaint();
	}
}
